@@ -1,6 +1,7 @@
 #![feature(allocator_api)]
 #![feature(const_generics)]
 #![feature(const_evaluatable_checked)]
+
 #![warn(unsafe_op_in_unsafe_fn)]
 #![deny(missing_docs)]
 #![allow(incomplete_features)]
@@ -32,7 +33,7 @@ use bytemuck::Pod;
 use chunk_iter::ChunkIter;
 use core::slice;
 use std::{
-    alloc::{AllocError, Allocator, Global, Layout},
+    alloc::{AllocError, Allocator, Layout},
     cmp::Ordering,
     fmt::Display,
     marker::PhantomData,
@@ -68,28 +69,28 @@ impl Display for Length {
 
 /// Implementation detail: Do not use
 #[derive(Copy, Clone, Debug)]
-pub struct AlignmentCorrectorAllocator<I, O> {
-    allocator: Global,
+pub struct AlignmentCorrectorAllocator<I, O, A: Allocator> {
+    allocator: A,
     ptr: *const O,
     phantom: PhantomData<I>,
 }
-impl<I, O> AlignmentCorrectorAllocator<I, O> {
-    fn new(ptr: *const O) -> Self {
+impl<I, O, A: Allocator> AlignmentCorrectorAllocator<I, O, A> {
+    fn new(ptr: *const O, allocator: A) -> Self {
         Self {
-            allocator: Global::default(),
+            allocator,
             ptr,
             phantom: PhantomData::default(),
         }
     }
-    fn new_null() -> Self {
+    fn new_null(allocator: A) -> Self {
         Self {
-            allocator: Global::default(),
+            allocator,
             ptr: ptr::null(),
             phantom: PhantomData::default(),
         }
     }
 }
-unsafe impl<I, O> Allocator for AlignmentCorrectorAllocator<I, O> {
+unsafe impl<I, O, A: Allocator> Allocator for AlignmentCorrectorAllocator<I, O, A> {
     #[inline]
     fn allocate(&self, layout: Layout) -> Result<NonNull<[u8]>, AllocError> {
         self.allocator.allocate(layout)
@@ -159,45 +160,59 @@ unsafe impl<I, O> Allocator for AlignmentCorrectorAllocator<I, O> {
     }
 }
 
-unsafe fn from_raw_parts<I, O>(
+unsafe fn from_raw_parts<I, O, A: Allocator>(
     old_ptr: *mut O,
     length: usize,
     capacity: usize,
-) -> Vec<O, AlignmentCorrectorAllocator<I, O>> {
+    allocator: A,
+) -> Vec<O, AlignmentCorrectorAllocator<I, O, A>> {
     // SAFETY: the caller uploads the constrants for from_raw_parts except for the alignment of the
     // allocation that created the old pointer
     unsafe {
-        Vec::<O, AlignmentCorrectorAllocator<I, O>>::from_raw_parts_in(
+        Vec::<O, AlignmentCorrectorAllocator<I, O, A>>::from_raw_parts_in(
             old_ptr,
             length,
             capacity,
-            AlignmentCorrectorAllocator::<I, O>::new(old_ptr),
+            AlignmentCorrectorAllocator::<I, O, A>::new(old_ptr, allocator),
         )
     }
 }
-/// Same as `transmute_vec` but in case of an error it copies instead.
-/// If it's over the length it removes whatever doesn't fit.
-pub fn transmute_vec_may_copy<I: Pod, O: Pod>(
-    input: Vec<I>,
-) -> Vec<O, AlignmentCorrectorAllocator<I, O>>
+
+/// Whether or not a copy occured.
+/// Also the copy variant doesn't have the custom allocator, and is therefore one usize smaller.
+pub enum CopyNot<I, O, A: Allocator> {
+    /// Copy occured
+    Copy(Vec<O, A>),
+    /// There was no copy.
+    Not(Vec<O, AlignmentCorrectorAllocator<I, O, A>>),
+}
+
+/// [`transmute_vec_may_copy`] but it tells you whether or not a copy occured and returns a normal
+/// vec if it doesn't.
+pub fn transmute_vec_copy_enum<I: Pod, O: Pod, A: Allocator>(input: Vec<I, A>) -> CopyNot<I, O, A>
 where
-    [(); mem::size_of::<O>()]: ,
+    [(); mem::size_of::<O>()]: , // todo: see if i can remove this where
 {
     match transmute_vec(input) {
-        Ok(x) => x,
+        Ok(x) => CopyNot::Not(x),
         Err((old_vec, err)) => match err {
             TransmuteError::Alignment => {
-                // let mut new_vec = Vec::<u8>::new();
-                // for x in old_vec {
-                //     new_vec.extend_from_slice(bytemuck::bytes_of(&x))
-                // }
-                let (ptr, length) = (old_vec.as_ptr(), old_vec.len());
+                let (ptr, length, capacity, allocator) = {
+                    let mut me = ManuallyDrop::new(old_vec);
+                    (me.as_mut_ptr(), me.len(), me.capacity(), unsafe {
+                        ptr::read(me.allocator())
+                    })
+                };
+                
 
                 // SAFETY: the ptr comes from a vec and the length is calcuated properly
                 let bytes_slice = unsafe {
                     slice::from_raw_parts(ptr.cast::<u8>(), length * mem::size_of::<I>())
                 };
-                let mut return_vec = Vec::new_in(AlignmentCorrectorAllocator::new_null());
+                let mut return_vec = Vec::with_capacity_in(
+                    (length * mem::size_of::<I>()) / mem::size_of::<O>(),
+                    allocator,
+                );
                 for x in bytes_slice
                     .iter()
                     .copied()
@@ -206,24 +221,84 @@ where
                     // SAFETY: O is Pod and the array is the same length as the type.
                     return_vec.push(unsafe { mem::transmute_copy(&x) })
                 }
-                return_vec
+                // freeing memory
+                // SAFETY: this size and align come from a vec and the allocator is the same one
+                // that allocated the memory. i dont have to call drop because its Pod
+                unsafe { 
+                    let align = mem::align_of::<I>();
+                    let size = mem::size_of::<I>() * capacity;
+                    let layout = Layout::from_size_align_unchecked(size, align) ;
+                    return_vec.allocator().deallocate(NonNull::new_unchecked(ptr.cast()), layout);
+                };
+                CopyNot::Copy(return_vec)
             }
             TransmuteError::Capacity | TransmuteError::Length => {
-                let (ptr, length) = (old_vec.as_ptr(), old_vec.len());
+                let (ptr, length, capacity, allocator) = {
+                    let mut me = ManuallyDrop::new(old_vec);
+                    (me.as_mut_ptr(), me.len(), me.capacity(), unsafe {
+                        ptr::read(me.allocator())
+                    })
+                };
 
                 // SAFETY: the divide rounds down so the length is correct
-                unsafe {
-                    slice::from_raw_parts(
-                        ptr.cast::<O>(),
-                        (length * mem::size_of::<I>()) / mem::size_of::<O>(),
-                    )
-                }
-                .to_vec_in(AlignmentCorrectorAllocator::new_null())
+                let return_vec = 
+                    unsafe {
+                        slice::from_raw_parts(
+                            ptr.cast::<O>(),
+                            (length * mem::size_of::<I>()) / mem::size_of::<O>(),
+                        )
+                    }
+                    .to_vec_in(allocator);
+                
+                // freeing memory
+                // SAFETY: this size and align come from a vec and the allocator is the same one
+                // that allocated the memory. i dont have to call drop because its Pod
+                unsafe { 
+                    let align = mem::align_of::<I>();
+                    let size = mem::size_of::<I>() * capacity;
+                    let layout = Layout::from_size_align_unchecked(size, align) ;
+                    return_vec.allocator().deallocate(NonNull::new_unchecked(ptr.cast()), layout);
+                };
+                CopyNot::Copy(return_vec)
+
             }
         },
     }
 }
 
+/// Same as `transmute_vec` but in case of an error it copies instead.
+/// If it's over the length it removes whatever doesn't fit.
+/// You may want to use [`transmute_vec_copy_enum`].
+pub fn transmute_vec_may_copy<I: Pod, O: Pod, A: Allocator>(
+    input: Vec<I, A>,
+) -> Vec<O, AlignmentCorrectorAllocator<I, O, A>>
+where
+    [(); mem::size_of::<O>()]: ,
+{
+    match transmute_vec_copy_enum(input) {
+        CopyNot::Copy(x) => {
+            let (ptr, length, capacity, allocator) = {
+                let mut me = ManuallyDrop::new(x);
+                (me.as_mut_ptr(), me.len(), me.capacity(), unsafe {
+                    ptr::read(me.allocator())
+                })
+            };
+
+            // SAFETY: comes directly from vec and AlignmentCorrectorAllocator::new_null
+            // doesn't actually do anything
+            unsafe {
+                Vec::from_raw_parts_in(
+                    ptr,
+                    length,
+                    capacity,
+                    AlignmentCorrectorAllocator::new_null(allocator),
+                )
+            }
+        }
+        CopyNot::Not(x) => x,
+    }
+}
+
 /// Allows transmuting of a Vec to another vec of a different size, with 0 copies.
 /// # Example
 /// ```
@@ -265,12 +340,14 @@ where
 /// # See also
 /// - [`transmute_vec_may_copy`]
 #[allow(clippy::type_complexity)]
-pub fn transmute_vec<I: Pod, O: Pod>(
-    input: Vec<I>,
-) -> Result<Vec<O, AlignmentCorrectorAllocator<I, O>>, (Vec<I>, TransmuteError)> {
-    let (ptr, length, capacity) = {
+pub fn transmute_vec<I: Pod, O: Pod, A: Allocator>(
+    input: Vec<I, A>,
+) -> Result<Vec<O, AlignmentCorrectorAllocator<I, O, A>>, (Vec<I, A>, TransmuteError)> {
+    let (ptr, length, capacity, allocator) = {
         let mut me = ManuallyDrop::new(input);
-        (me.as_mut_ptr(), me.len(), me.capacity())
+        (me.as_mut_ptr(), me.len(), me.capacity(), unsafe {
+            ptr::read(me.allocator())
+        })
     };
 
     match mem::size_of::<I>().cmp(&mem::size_of::<O>()) {
@@ -278,19 +355,19 @@ pub fn transmute_vec<I: Pod, O: Pod>(
             if ptr.align_offset(mem::align_of::<O>()) != 0 {
                 Err((
                     // SAFETY: this came directly from a vec
-                    unsafe { Vec::from_raw_parts(ptr, length, capacity) },
+                    unsafe { Vec::from_raw_parts_in(ptr, length, capacity, allocator) },
                     TransmuteError::Alignment,
                 ))
             } else if (length * mem::size_of::<I>()) % mem::size_of::<O>() != 0 {
                 Err((
                     // SAFETY: this came directly from a vec
-                    unsafe { Vec::from_raw_parts(ptr, length, capacity) },
+                    unsafe { Vec::from_raw_parts_in(ptr, length, capacity, allocator) },
                     TransmuteError::Length,
                 ))
             } else if (capacity * mem::size_of::<I>()) % mem::size_of::<O>() != 0 {
                 Err((
                     // SAFETY: this came directly from a vec
-                    unsafe { Vec::from_raw_parts(ptr, length, capacity) },
+                    unsafe { Vec::from_raw_parts_in(ptr, length, capacity, allocator) },
                     TransmuteError::Capacity,
                 ))
             } else {
@@ -302,6 +379,7 @@ pub fn transmute_vec<I: Pod, O: Pod>(
                         ptr.cast(),
                         (length * mem::size_of::<I>()) / mem::size_of::<O>(),
                         (capacity * mem::size_of::<I>()) / mem::size_of::<O>(),
+                        allocator,
                     )
                 })
             }
@@ -309,11 +387,11 @@ pub fn transmute_vec<I: Pod, O: Pod>(
         Ordering::Equal => {
             if ptr.align_offset(mem::align_of::<O>()) == 0 {
                 // SAFETY: its aligned and thats all that matters
-                Ok(unsafe { from_raw_parts(ptr.cast(), length, capacity) })
+                Ok(unsafe { from_raw_parts(ptr.cast(), length, capacity, allocator) })
             } else {
                 Err((
                     // SAFETY: this came directly from a vec
-                    unsafe { Vec::from_raw_parts(ptr, length, capacity) },
+                    unsafe { Vec::from_raw_parts_in(ptr, length, capacity, allocator) },
                     TransmuteError::Alignment,
                 ))
             }
@@ -380,7 +458,7 @@ mod tests {
     #[test]
     fn wrong_length() {
         let input: Vec<u8> = vec![1, 2, 3];
-        match transmute_vec::<_, u16>(input) {
+        match transmute_vec::<_, u16, _>(input) {
             Ok(_) => panic!(),
             Err((_, err)) => match err {
                 TransmuteError::Alignment | TransmuteError::Length => (),
